import logging
import requests
from urllib.parse import urlparse, urlsplit
from .resolvers import generic_resolver


class UrlUnshortener:
    def __init__(self, tricks=False):
        self.services = ["bit.ly", "is.gd", "t.co", "tinyurl.com", "tiny.cc"]
        if tricks == False:
            self.tricks = False
        else:
            self.tricks = True

    def resolve_short(self, url, imeout=None):
        """
        Resolve shortened URL to a target URL. If the URL could not be resolved,
        return None. It would be good to validate that this is actually a url
        here, but I can't seem to easily get that done without some other
        dependencies and I'm trying to keep this simple.

        :argument url: the url to resolve

        :argumet timeout: the max tix to wait for the URL to resolve in seconds
            There's currently no way of telling if a request failed due to
            a timeout or because the URL could not be resolved!

        :returns: the resolved URL. None if URL could not be resolved

        """

        parts = urlsplit(url)
        if not parts.scheme and not parts.hostname:
            parts = urlsplit("https://" + url)
            url = parts.join()
        result = generic_resolver(url, self.tricks)
        return result

    def is_shortened(self, url):
        """Check if the url appears to be shortened, based on the services
        whitelist. **Note:** This will be a best-effort thing, as the list
        if services has to be kept up to date. Also note that valid URLs on
        shortening services (like bit.ly/apidocs) will be assumed to be a
        shortened url.

        :argument url: The URL to check

        :returns: true or false
        """
        parts = urlsplit(url)
        if parts.hostname in self.services:
            if not parts.scheme and not parts.hostname:
                parts = urlsplit("http://" + url)
            return bool(parts.hostname in self.services and parts.path)
        else:
            logging.debug('[-] Service at {} is not supported by this function, but may resolve'.format(url))
            try:
                r = requests.head(url)
                if r.headers['Location'] != url:
                    return True
            except:
                return False




import unittest
import logging
from urlunshort3 import UrlUnshortener

# all of these should resolve to https://github.com/remotephone/urlunshort3

class TestResolve(unittest.TestCase):
    shortener = UrlUnshortener()

    def test_generic(self):
        logging.info("Testing generic url resolution")
        shortener = UrlUnshortener(tricks=True)
        urls = ["https://bit.ly/2nBRa1c", "https://tinyurl.com/yyucc4ph", "https://is.gd/gPqAfb", "https://t.co/202LlppKBL", "https://tiny.cc/d0efez"]
        for url in urls:
            dst = shortener.resolve_short(url)
            self.assertEqual(dst, "https://github.com/remotephone/urlunshort3")

    def test_404(self):
        logging.info("Testing known 404")
        shortener = UrlUnshortener()
        assert shortener.resolve_short("http://example.org/adsfasd.fasdfasads,masdf") == None

    def test_bad_host(self):
        logging.info("Testing known bad host")
        shortener = UrlUnshortener()
        assert shortener.resolve_short("http://alewklfafadsfaewfadasfa/foo") == None


if __name__ == "__main__":
    unittest.main()



import setuptools
from distutils.core import setup
from pathlib import Path

thisdir = Path(__file__).parent
readme = thisdir / 'README.md'
with open(readme, encoding='utf-8') as f:
    long_description = f.read()



setup(
    name="urlunshort3",  # How you named your package folder (MyLib)
    packages=setuptools.find_packages(),
    version="0.2",  # Start with a small number and increase it with every change you make
    license="BSD",  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description="Take a URL, compare to known shortening services, and expand it. A python3 rewrite of https://pypi.org/project/urlunshort/",  # Give a short description about your library
    long_description=long_description,
    author="remotephone",  # Type in your name
    author_email="remotephone@gmail.com",  # Type in your E-Mail
    url="https://github.com/user/reponame",  # Provide either the link to your github or to your website
    keywords=["urlshorten", "expand", "security"],  # Keywords that define your package best
    install_requires=["requests>=2.22.0"],  # I get to this in a second
    classifiers=[
        "Development Status :: 3 - Alpha",  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
        "Intended Audience :: Information Technology",  # Define that your audience are developers
        "Topic :: Internet :: WWW/HTTP",
        "License :: OSI Approved :: BSD License",  # Again, pick a license
        "Programming Language :: Python :: 3.6",
    ],
)



"""
This module contains resolvers for url shorteners. A resolver takes a netloc
(host[:port] and path) and returns a URL as a string.
"""
import logging
import requests
from urllib.parse import urlparse, urlsplit
from functools import wraps


def handle_tricks(url, tricks):
    # Some URL shorteners don't respond correctly to a head request, this may help
    if tricks == True:
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36"
        }
        r = requests.get(url, headers=headers)
        if r.status_code in [301, 303]:
            logging.info('"{}" returned a {} response'.format(url, r.status_code))
            return r.headers['Location']
        else:
            logging.debug(
                'URL "{}" returned a {} status code'.format(url, r.status_code)
            )
            return r.url
    else:
        return None


def _io_error_handling(fun):
    """
    Catch connection errors for requests
    """

    @wraps(fun)
    def wrapped(*args, **kwargs):
        try:
            return fun(*args, **kwargs)
        except requests.ConnectionError:
            return None

    return wrapped


@_io_error_handling
def generic_resolver(url, tricks, timeout=None):
    """
    Generic url fetcher that assumes the target service will response sanely
    to a HEAD request for the url.

    returns the full url or None if the url could not be resolved.
    """
    request_error = range(400, 600)

    if timeout:
        r = requests.head(url, timeout=timeout)
    else:
        r = requests.head(url)
    redirect_codes = range(300, 400)

    if r.status_code in request_error:
        logging.info(
            '"{}" returned a {} status code for value {}'.format(
                url, r.status_code, r.url
            )
        )
        if urlsplit(r.url).netloc == urlsplit(url).netloc:
            response = handle_tricks(url, tricks)
            return response
        else:
            return r.url
    elif r.status_code in request_error:
        logging.info('URL "{}" returned an unhandled response'.format(url))
        return None
    elif r.status_code in redirect_codes:
        return r.headers["Location"]
    else:
        logging.error('{} returned a {} response'.format(url, r.status_code))
        return None


    return None



import unittest
from urlunshort3 import UrlUnshortener


class TestResolve(unittest.TestCase):
    def test_is_shortened(self):
        shortener = UrlUnshortener()
        for service in shortener.services:
            assert shortener.is_shortened("http://{}/asdf".format(service)), "http://{}/asdf".format(service)
            assert not shortener.is_shortened("http://{}".format(service)), "http://{}".format(service)


if __name__ == "__main__":
    unittest.main()
